C This program calculates the transformation strain during bainite/martensite transformation.
C This is done for any number of grains and all the active variants. It shows how any three 
C perpendicular directions are deformed due to the transformation. The shape deformation
C matrices are calculated taking various habit planes and displacement directions as input. 
C The program also takes into account any amount of
C retained austenite in the material. It can be shown with the help of this program that the strain
C due to transformation is not isotropic in nature. It can also be shown that as the number of 
C variants reduces (due to variant selection under stress) the strain increases. If all the variants
C are operative then the deformation due to transformation becomes equal to the volume strain.

C The two major inputs are the habit planes and displacement 
C directions from two files.The program also takes the rotation matrix as input that gives the 
C orientation relationship between various austenite grains. So this way we can also introduce bias 
C in the microstructure prior to transformation.

C The applied stress here is represented by a stress tensor, which is useful to 
C deal with mixed mode stress. The strain due to transformation is calculated along
C three principle axes in which load is applied. 


C
C Developped by Saurabh Kundu, April, 2006, Cambridge.
C

      PROGRAM TRANS_PLAST_TENSOR
      
      IMPLICIT NONE
      
      INTEGER DMAX,RW,RWW,RR,RS
      DOUBLE PRECISION PI
      
      PARAMETER (DMAX=72,RW=99,RR=66,RS=67,PI=3.14159)
	      
      DOUBLE PRECISION X_TEMP(3),Y_TEMP(3),Z_TEMP(3)
      DOUBLE PRECISION X_RES(3),Y_RES(3),Z_RES(3)
      DOUBLE PRECISION ALL_SHAPE(DMAX,3),THETA(24),PHI(24)
      DOUBLE PRECISION HABIT(24,3),DIRECTION(24,3)
      DOUBLE PRECISION P(3),DIREC(3),N(3),K(3),SH_TEMP(3),D(3)
      DOUBLE PRECISION X_ARBIT(3),Y_ARBIT(3),Z_ARBIT(3),DIRECT(3)
      DOUBLE PRECISION PLASTIC_X,PLASTIC_Y,PLASTIC_Z,ALL_INFO(24,4)
      DOUBLE PRECISION TRANS_MATRIX(3,3),TEMP,SIGMA(3),BETA
      DOUBLE PRECISION TAO_TEMP(3),TAO(DMAX,3),SHEAR(DMAX,3)
      DOUBLE PRECISION SIGMA_START(3),FRAC,MAX_VAR
      DOUBLE PRECISION STRAIN_X,STRAIN_Y,STRAIN_Z
      DOUBLE PRECISION ALPH_NEW,P_INTO_D,U(24),STRESS,DET
      DOUBLE PRECISION SIGMA_TEMP(3),A,TRANS_MATRIX_INV(3,3)
      DOUBLE PRECISION X(3),Y(3),Z(3),X_FINAL(3),Y_FINAL(3),Z_FINAL(3)
      DOUBLE PRECISION STRESS_TENSOR(3,3),TRANS_MATRIX_TRANSP(3,3)
      DOUBLE PRECISION DEF_MAT(3,3),MAX_ENG,SH,DI,TOT,TRACK_ENG(24)
      INTEGER I,J,NO_GRAINS,ISEED,REPEAT,G,H,F,VAR,LOAD,II
      INTEGER TEXTURE,ENG_TAG
	
      ISEED=TIME()
      CALL SRAND(ISEED)
      
c      WRITE(*,*) 'X = ',X(1),X(2),X(3)
c      WRITE(*,*) 'Y = ',Y(1),Y(2),Y(3)
c      WRITE(*,*) 'Z = ',Z(1),Z(2),Z(3)

      OPEN(RW,FILE='trans-plasticity.txt',STATUS='OLD')
C&      ,ACCESS='APPEND')
      
      WRITE(*,*) 'WRITE THE NUMBER OF GRAINS: '
      READ(*,*) NO_GRAINS
      
c      NO_GRAINS=500
          
      WRITE(*,*) 'FRACTION OF MARTENSITE/BAINITE: '
      READ(*,*) FRAC
      
      FRAC=0.63
      
C      WRITE(*,*) 'MAXIMUM NO OF VARIANTS : '
C      READ(*,*) MAX_VAR
      
      MAX_VAR=24
      
      WRITE(*,*) 'INITIAL TEXTURE: '
      WRITE(*,*) '0=RANDOM'
      WRITE(*,*) '1=COPPER'
      WRITE(*,*) '2=CUBE'
      WRITE(*,*) '3=GOSS'
      WRITE(*,*) '4={111}<110>'
      WRITE(*,*) '5=RAW'
      TEXTURE=0
c      READ(*,*) TEXTURE
      
      IF(TEXTURE .EQ. 0) THEN
           CALL GENERATE_RANDOM()
      END IF

      OPEN(RS,FILE='sh-vol.txt',STATUS='OLD')
      
      READ(RS,*) TOT,SH,DI
      WRITE(*,*) 'TOTAL DEFORMATION: ',TOT
      WRITE(*,*) 'TOTAL DEFORMATION: ',SH
      WRITE(*,*) 'VOLUME DEFORMATION: ',DI
      
      CLOSE(RS)
         
      CALL READING(X,Y,Z,SIGMA,SIGMA_START,HABIT,DIRECTION,LOAD,
&          STRESS_TENSOR)
      
      DO 8 I=1,24
         PHI(I)=0
         THETA(I)=0
8     CONTINUE
      
      MAX_ENG=0
      
      DO 9 II=1,MAX_VAR
      
         IF(TEXTURE .EQ. 0) THEN
C           CALL GENERATE_RANDOM()
          OPEN(RR,FILE='random-matrix-input-plast.txt',STATUS='OLD')
         ELSE IF(TEXTURE .EQ. 1) THEN
          OPEN(RR,FILE='matrix-bias-texture-cu-500.txt',STATUS='OLD')
         ELSE IF(TEXTURE .EQ. 2) THEN
          OPEN(RR,FILE='matrix-bias-texture-cube-500.txt',STATUS='OLD')
         ELSE IF(TEXTURE .EQ. 3) THEN
          OPEN(RR,FILE='matrix-bias-texture-goss-500.txt',STATUS='OLD')
         ELSE IF(TEXTURE .EQ. 4) THEN
          OPEN(RR,FILE='matrix-bias-texture-110-111-500.txt'
&          ,STATUS='OLD')
         ELSE IF(TEXTURE .EQ. 5) THEN
          OPEN(RR,FILE='matrix-bias-texture-raw.txt',STATUS='OLD')
         END IF
         
         DO 5 I=1,3
         X_FINAL(I)=0
         Y_FINAL(I)=0
         Z_FINAL(I)=0
5	    CONTINUE
 
c        TRANS_MATRIX(1,1)=1.0
c        TRANS_MATRIX(1,2)=0.0
c        TRANS_MATRIX(1,3)=0.0
c      
c        TRANS_MATRIX(2,1)=0.0
c        TRANS_MATRIX(2,2)=1.0
c        TRANS_MATRIX(2,3)=0.0
c      
c        TRANS_MATRIX(3,1)=0.0
c        TRANS_MATRIX(3,2)=0.0
c        TRANS_MATRIX(3,3)=1.0
c        
        
              
c        CALL CALCULATE_INVERSE(TRANS_MATRIX,TRANS_MATRIX_INV)
        
c        CALL CALCULATE_TRANSPOSE(TRANS_MATRIX,TRANS_MATRIX_TRANSP)

        READ(66,*) TRANS_MATRIX(1,1)
        READ(66,*) TRANS_MATRIX(1,2)
        READ(66,*) TRANS_MATRIX(1,3)
        READ(66,*) TRANS_MATRIX(2,1)
        READ(66,*) TRANS_MATRIX(2,2)
        READ(66,*) TRANS_MATRIX(2,3)
        READ(66,*) TRANS_MATRIX(3,1)
        READ(66,*) TRANS_MATRIX(3,2)
        READ(66,*) TRANS_MATRIX(3,3)
         
               
         CALL CALCULATE_INVERSE(TRANS_MATRIX,TRANS_MATRIX_INV)
        
         CALL CALCULATE_TRANSPOSE(TRANS_MATRIX,TRANS_MATRIX_TRANSP)
              
        DO 10 I=1,NO_GRAINS
                  
         DO 11 H=1,3
            DO 12 J=1,3
               DEF_MAT(H,J)=0
12          CONTINUE
11       CONTINUE
         
C         WRITE(*,*) 'TRANS_MATRIX'
C        CALL WRITEMAT(TRANS_MATRIX)   
         
         CALL CAL_DEFMAT(TRANS_MATRIX,TRANS_MATRIX_INV,
&                 STRESS_TENSOR,DEF_MAT)
             
         DO 20 G=1,24
            
            DO 30 H=1,3
                P(H)=HABIT(G,H)
30          CONTINUE

            CALL TENSOR_CALC(SIGMA,DEF_MAT,P,STRESS)
            
            CALL CAL_THETA(P,SIGMA,THETA,G)
              
             IF(LOAD .EQ. 1) THEN
               CALL CROSS_PROD(P,SIGMA,K)
             ELSE
               CALL CROSS_PROD(SIGMA,P,K)
             END IF
                         
             IF(THETA(G) .LT. PI/2) THEN
              BETA=PI/2-THETA(G)
             ELSE
c               BETA=PI/2-THETA(G)
               BETA=PI/2-(PI-THETA(G))
             ENDIF
         
             D(1)=0
             D(2)=0
             D(3)=COS(BETA)
         
C           WRITE(*,*) 'THE VALUE OF K = ', K(1), K(2), K(3)
         
             CALL CRAMER(TAO_TEMP,P,K,SIGMA,D)
         
             DO 40 H=1,3
               TAO(G,H)=TAO_TEMP(H)
   40        CONTINUE     
   
c             WRITE(*,*) 'TAOVEC=',TAO_TEMP(1),TAO_TEMP(2),TAO_TEMP(3)
                
  
C ** Calculating the unit vector in the direction of maximum shear stress ****

C ** Calculating the unit vector in the direction of shear stress on the HP ** 
       
             DO 50 H=1,3
                DIREC(H)=DIRECTION(G,H)
   50        CONTINUE
   
             CALL CROSS_PROD(DIREC,P,N)
            
             P_INTO_D=P(1)*DIREC(1)+P(2)*DIREC(2)+P(3)*DIREC(3)
            
             P_INTO_D=ACOS(P_INTO_D)
            
             IF(P_INTO_D .LE. PI/2) THEN
               ALPH_NEW=PI/2-P_INTO_D
             ELSE
               ALPH_NEW=P_INTO_D-PI/2
             END IF
               
             D(1)=0
             D(2)=0
             D(3)=COS(ALPH_NEW)
         
             CALL CRAMER(SH_TEMP,P,N,DIREC,D)
         
             DO 60 H=1,3
               SHEAR(G,H)=SH_TEMP(H)
   60        CONTINUE  
   
C             WRITE(*,*) 'SHEAR=',SH_TEMP(1),SH_TEMP(2),SH_TEMP(3)
C ******************************************************************************

C
C ************Calculation of angle phi and mechanical driving force ***********
C
             CALL CALCULATE_PHI(TAO_TEMP,SH_TEMP,PHI,G)
                                  
             CALL CALCULATE_MECH_ENG(THETA,PHI,BETA,U,G,STRESS,SH,DI)
            
C *****************************************************************************
   
   20    CONTINUE
         
C ***** Calculation of the equivalent gamma pole for a given alpha pole *******  

         CALL SORTING(THETA,PHI,U,ALL_INFO)
         
c         IF(II .EQ. 1) THEN
c            DO 21 G=1,24
c               TRACK_ENG(G)=TRACK_ENG(G)+ALL_INFO(G,4) 
c21          CONTINUE
c         END IF
        
c         DO 22 G=1,24
c            WRITE(*,888) ALL_INFO(G,1),ALL_INFO(G,2)*180/PI,
c&            ALL_INFO(G,3)*180/PI,ALL_INFO(G,4) 
c888         FORMAT(F8.0,3F10.3)  
c22       CONTINUE
c         write(*,*) ''
         DO 23 G=1,3
           X_RES(G)=0
           Y_RES(G)=0
           Z_RES(G)=0
   23    CONTINUE
         
          DO 25 G=1,3
         
            X_TEMP(G)=0
            Y_TEMP(G)=0
            Z_TEMP(G)=0
   25    CONTINUE
        
           DO 111 G=1,3
            DO 121 H=1,3
               X_TEMP(G)=X_TEMP(G)+TRANS_MATRIX_INV(G,H)*X(H)
               Y_TEMP(G)=Y_TEMP(G)+TRANS_MATRIX_INV(G,H)*Y(H)
               Z_TEMP(G)=Z_TEMP(G)+TRANS_MATRIX_INV(G,H)*Z(H)
  121       CONTINUE
  111    CONTINUE
           
         CALL CALCULATE_TRANS_PLAST(HABIT,DIRECTION,ALL_INFO,STRESS,
&         TRANS_MATRIX,NO_GRAINS,X_TEMP,X_RES,Y_TEMP,Y_RES,Z_TEMP,
&         Z_RES,FRAC,II,LOAD,TOT)  
         
         DO 101 G=1,3
            X_FINAL(G)=X_FINAL(G)+X_RES(G)
            Y_FINAL(G)=Y_FINAL(G)+Y_RES(G)
            Z_FINAL(G)=Z_FINAL(G)+Z_RES(G)
  101    CONTINUE
         
         IF(I .NE. NO_GRAINS) THEN
            READ(66,*) TRANS_MATRIX(1,1)
            READ(66,*) TRANS_MATRIX(1,2)
            READ(66,*) TRANS_MATRIX(1,3)
            READ(66,*) TRANS_MATRIX(2,1)
            READ(66,*) TRANS_MATRIX(2,2)
            READ(66,*) TRANS_MATRIX(2,3)
            READ(66,*) TRANS_MATRIX(3,1)
            READ(66,*) TRANS_MATRIX(3,2)
            READ(66,*) TRANS_MATRIX(3,3)
          END IF
              
            CALL CALCULATE_INVERSE(TRANS_MATRIX,TRANS_MATRIX_INV)

            CALL CALCULATE_TRANSPOSE(TRANS_MATRIX,TRANS_MATRIX_TRANSP)
         
         DO 100 G=1,3
         
            X_TEMP(G)=0
            Y_TEMP(G)=0
            Z_TEMP(G)=0
  100    CONTINUE
      
         DO 110 G=1,3
            DO 120 H=1,3
               X_TEMP(G)=X_TEMP(G)+TRANS_MATRIX_INV(G,H)*X(H)
               Y_TEMP(G)=Y_TEMP(G)+TRANS_MATRIX_INV(G,H)*Y(H)
               Z_TEMP(G)=Z_TEMP(G)+TRANS_MATRIX_INV(G,H)*Z(H)
  120       CONTINUE
  110    CONTINUE
         
  
         A=(SQRT(X_TEMP(1)**2+X_TEMP(2)**2
&                              +X_TEMP(3)**2))

         DO 131 H=1,3
            X_TEMP(H)=X_TEMP(H)/A
  131    CONTINUE
  
         A=(SQRT(Y_TEMP(1)**2+Y_TEMP(2)**2
&                              +Y_TEMP(3)**2))

         DO 132 H=1,3
            Y_TEMP(H)=Y_TEMP(H)/A
  132    CONTINUE
  
         A=(SQRT(Z_TEMP(1)**2+Z_TEMP(2)**2
&                              +Z_TEMP(3)**2))

         DO 133 H=1,3
            Z_TEMP(H)=Z_TEMP(H)/A
  133    CONTINUE
      
      IF(II .EQ. 1) THEN
         MAX_ENG=MAX_ENG+ALL_INFO(1,4)
      END IF
      
   10 CONTINUE
   
      STRAIN_X=SQRT(X_FINAL(1)**2+X_FINAL(2)**2+X_FINAL(3)**2)
      STRAIN_Y=SQRT(Y_FINAL(1)**2+Y_FINAL(2)**2+Y_FINAL(3)**2)
      STRAIN_Z=SQRT(Z_FINAL(1)**2+Z_FINAL(2)**2+Z_FINAL(3)**2)
      
      WRITE(*,*) 'No of variant working : ',II
      
      WRITE(*,*) 'X_INITIAL=',X(1),X(2),X(3)
      WRITE(*,*) 'Y_INITIAL=',Y(1),Y(2),Y(3)
      WRITE(*,*) 'Z_INITIAL=',Z(1),Z(2),Z(3)
      
      WRITE(*,*) 'X_FINAL',X_FINAL(1),X_FINAL(2),X_FINAL(3)     
      WRITE(*,*) 'Y_FINAL',Y_FINAL(1),Y_FINAL(2),Y_FINAL(3)      
      WRITE(*,*) 'Z_FINAL',Z_FINAL(1),Z_FINAL(2),Z_FINAL(3)
      WRITE(*,*) 'STRAIN',STRAIN_X,STRAIN_Y,STRAIN_Z
      WRITE(99,889) II,STRAIN_X,STRAIN_Y,STRAIN_Z
889   FORMAT(I5,3F10.6)    

      
       
      CLOSE(RR)
          
9     CONTINUE 

       DO 41 G=1,24
               TRACK_ENG(G)=TRACK_ENG(G)/NO_GRAINS 
c               WRITE(*,*) 'VARIANT RANK= ',G,' INT. ENG.=',TRACK_ENG(G)
41          CONTINUE


      MAX_ENG=MAX_ENG/NO_GRAINS

      WRITE(*,*) 'MAXIMUM INTERACTION ENERGY:',MAX_ENG
      
      CLOSE(RW)
     
      END
                                  
C ********************************************************************************** C
C ********************************************************************************** C

      SUBROUTINE READING(X,Y,Z,SIGMA,SIGMA_START,HABIT,DIRECTION
&      ,LOAD,STRESS_TENSOR)
      
      IMPLICIT NONE
      
      INTEGER DMAX,RH,tag
      DOUBLE PRECISION PI
      PARAMETER (DMAX=72,RH=19,PI=3.14159)
      
      DOUBLE PRECISION X(3),Y(3),Z(3)
      DOUBLE PRECISION A,HABIT(24,3),DIRECTION(24,3)
      DOUBLE PRECISION SIGMA(3),SIGMA_START(3)
      DOUBLE PRECISION TEMP_SHAPE(3,3),INV_TEMP_SHAPE(3,3)
      DOUBLE PRECISION STRESS_TENSOR(3,3)
      INTEGER I,FLAG,K,S,J,LOAD
      
C      ISEED=TIME()
C      CALL SRAND(ISEED)
      
      FLAG=0
      
      OPEN(RH,FILE='habit-other.txt',STATUS='OLD')
      
      DO 10 I=1,24
         READ(RH,*) HABIT(I,1), HABIT(I,2), HABIT(I,3)
   10 CONTINUE
      
      CLOSE(RH)
      
      DO 11 I=1,24
c         write(*,*) HABIT(I,1), HABIT(I,2), HABIT(I,3)
11    CONTINUE
      
      OPEN(RH,FILE='dis-direction-other.txt',STATUS='OLD')
      
      DO 20 I=1,24
         READ(RH,*) DIRECTION(I,1), DIRECTION(I,2), DIRECTION(I,3)
   20 CONTINUE
   
      CLOSE(RH)
c      write(*,*) ''
      DO 12 I=1,24
c         write(*,*) DIRECTION(I,1), DIRECTION(I,2), DIRECTION(I,3)
12    CONTINUE
   
      
C ***** GENERATING DIRECTION OF AN ARBITRARY UNIT VECTOR *******************C

      I=1
      
      DO 90 WHILE(I .LE. 3)
            A=RAND()
            X(I)=A
            I=I+1
   90    CONTINUE
   
      A=(SQRT(X(1)**2+X(2)**2+X(3)**2))

      DO 100 I=1,3
            X(I)=X(I)/A
  100 CONTINUE
   
      
     
C ***********************************************************************************************C
C ***** GENERATING SECOND ARBITRARY DIRECTION PERPENDICULAR TO THE FIRST ONE ***** C                                                 ***** C

      IF(X(3) .NE. 0 .AND. FLAG .EQ. 0) THEN  
        WRITE(*,*)'IN 3'
        Z(1)=2*RAND()-1
        Z(2)=2*RAND()-1
        Z(3)=-(X(1)*Z(1)+X(2)*Z(2))/X(3)
        FLAG = 1
      END IF
      
      IF(X(2) .NE. 0 .AND. FLAG .EQ. 0) THEN  
        WRITE(*,*)'IN 2'
        Z(1)=2*RAND()-1
        Z(3)=2*RAND()-1
        Z(2)=-(X(1)*Z(1)+X(3)*Z(3))/X(2)
        FLAG = 1
      END IF
      
      IF(X(1) .NE. 0 .AND. FLAG .EQ. 0) THEN  
        WRITE(*,*)'IN 1'
        Z(2)=2*RAND()-1
        Z(3)=2*RAND()-1
        Z(1)=-(X(2)*Z(2)+X(3)*Z(3))/X(1)
        FLAG = 1
      END IF
      
      A=(SQRT(Z(1)**2+Z(2)**2+Z(3)**2))
      
      DO 110 I=1,3
            Z(I)=Z(I)/A
  110 CONTINUE
   
C ***** GENERATING THIRD ARBITRARY DIRECTION PERPENDICULAR TO THE FIRST TWO ***** C      
      CALL CROSS_PROD(X,Z,Y)
         
      
      
      WRITE(*,*) 'FRIST ELEMENT OF STRESS TENSOR:'
      READ(*,*) STRESS_TENSOR(1,1)
      
c      STRESS_TENSOR(1,1)=0.0
      STRESS_TENSOR(1,2)=0.0
      STRESS_TENSOR(1,3)=0.0
      
      STRESS_TENSOR(2,1)=0.0
      STRESS_TENSOR(2,2)=0.0
      STRESS_TENSOR(2,3)=0.0
      
      STRESS_TENSOR(3,1)=0.0
      STRESS_TENSOR(3,2)=0.0
      STRESS_TENSOR(3,3)=0.0
      
      TAG=0
      DO 111 I=1,3
         DO 112 J=1,3
            IF(STRESS_TENSOR(I,J) .NE. 0) THEN
               TAG=TAG+1
            END IF
112      CONTINUE
111   CONTINUE

      IF(TAG .NE. 0) THEN
        X(1)=1
        X(2)=0
        X(3)=0
      
        Y(1)=0
        Y(2)=1
        Y(3)=0
      
        Z(1)=0
        Z(2)=0
        Z(3)=1
       END IF
      
      CALL WRITEMAT(STRESS_TENSOR)
      
      WRITE(*,*) 'X=',X(1),X(2),X(3)
      WRITE(*,*) 'Y=',Y(1),Y(2),Y(3)
      WRITE(*,*) 'Z=',Z(1),Z(2),Z(3)
      WRITE(*,*) ''

C ***********************************************************************************************C
      END
      
C ********************************************************************************** C  
 
      SUBROUTINE CALCULATE_TRANS_PLAST(HABIT,DIRECTION,ALL_INFO,STRESS,
&         TRANS_MATRIX,NO_GRAINS,X_TEMP,X_RES,Y_TEMP,Y_RES,Z_TEMP,
&         Z_RES,FRAC,N,LOAD,TOT)  
      IMPLICIT NONE
      
      INTEGER DMAX,RH
      
      PARAMETER (DMAX=72,RH=19)
      
      INTEGER NO_GRAINS,NO,I,J,K,N,S,V,VAR,TEMP_V(24),SS,COMP,NN
      INTEGER LOAD
      DOUBLE PRECISION X_TEMP(3),X_RES(3),Y_TEMP(3),Y_RES(3)
      DOUBLE PRECISION HABIT(24,3),DIRECTION(24,3),TRANS_MATRIX(3,3)
      DOUBLE PRECISION ALL_INFO(24,4),STRESS,Z_TEMP(3),Z_RES(3)
      DOUBLE PRECISION P,M,SHAPE_DEF(3,3),XA(3),YA(3),ZA(3),FRAC
      DOUBLE PRECISION IDEN(3,3),PP,TOT
      
      
      DO 12 I=1,24
         TEMP_V(I)=0
   12 CONTINUE
         
      VAR=N
      COMP =0
      
      IDEN(1,1)=1
      IDEN(1,2)=0
      IDEN(1,3)=0
      
      IDEN(2,1)=0
      IDEN(2,2)=1
      IDEN(2,3)=0
      
      IDEN(3,1)=0
      IDEN(3,2)=0
      IDEN(3,3)=1
      
      M=TOT
      
c      WRITE(*,*) 'SH-',M
     
      DO 11 I=1,3
         XA(I)=0
         YA(I)=0
         ZA(I)=0
   11 CONTINUE
     
      P=N*NO_GRAINS/FRAC
      PP=NO_GRAINS/(1-FRAC)
      
      
      DO 10 I=1,N
         IF(STRESS .EQ. 0) THEN
            DO 13 WHILE(COMP .EQ. 0)
               V=RAND()*25
               COMP=1
               DO 14 SS=1,N
                  IF(V .EQ. TEMP_V(SS) .OR. V .EQ. 0) THEN
                     COMP=0
                  END IF
   14          CONTINUE
   13       CONTINUE
            TEMP_V(I)=V   
            COMP=0
         ELSE
            V=ALL_INFO(I,1)
        
         END IF
            
         SHAPE_DEF(1,1)=(1/P)*(1+M*DIRECTION(V,1)*HABIT(V,1))
         SHAPE_DEF(1,2)=(1/P)*M*DIRECTION(V,1)*HABIT(V,2)
         SHAPE_DEF(1,3)=(1/P)*M*DIRECTION(V,1)*HABIT(V,3)
         
         SHAPE_DEF(2,1)=(1/P)*M*DIRECTION(V,2)*HABIT(V,1)
         SHAPE_DEF(2,2)=(1/P)*(1+M*DIRECTION(V,2)*HABIT(V,2))
         SHAPE_DEF(2,3)=(1/P)*M*DIRECTION(V,2)*HABIT(V,3)
         
         SHAPE_DEF(3,1)=(1/P)*M*DIRECTION(V,3)*HABIT(V,1)
         SHAPE_DEF(3,2)=(1/P)*M*DIRECTION(V,3)*HABIT(V,2)
         SHAPE_DEF(3,3)=(1/P)*(1+M*DIRECTION(V,3)*HABIT(V,3))
         
c         WRITE(*,997) V,HABIT(V,1),HABIT(V,2),HABIT(V,3),
c&          DIRECTION(V,1),DIRECTION(V,2),DIRECTION(V,3)
c997      FORMAT('--------',I5//3F10.6/3F10.6/)

      DO 21 J=1,3
         DO 31 NN=1,3
               YA(J)=YA(J)+SHAPE_DEF(J,NN)*Y_TEMP(NN)
               XA(J)=XA(J)+SHAPE_DEF(J,NN)*X_TEMP(NN)
               ZA(J)=ZA(J)+SHAPE_DEF(J,NN)*Z_TEMP(NN)
   31    CONTINUE
   21 CONTINUE
   
   
            
c            DO 20 J=1,3
c               WRITE(*,996) SHAPE_DEF(J,1),SHAPE_DEF(J,2),
c&                        SHAPE_DEF(J,3)
c996            FORMAT(3F10.6)
c   20       CONTINUE
C            write(*,*) ''
      
     
      
   10 CONTINUE
   
      DO 22 J=1,3
         DO 33 NN=1,3
               XA(J)=XA(J)+(1/PP)*IDEN(J,NN)*X_TEMP(NN)
               YA(J)=YA(J)+(1/PP)*IDEN(J,NN)*Y_TEMP(NN)
               ZA(J)=ZA(J)+(1/PP)*IDEN(J,NN)*Z_TEMP(NN)
   33    CONTINUE
   22 CONTINUE
   
      DO 30 J=1,3
            DO 40 NN=1,3
               X_RES(J)=X_RES(J)+TRANS_MATRIX(J,NN)*XA(NN)
               Y_RES(J)=Y_RES(J)+TRANS_MATRIX(J,NN)*YA(NN)
               Z_RES(J)=Z_RES(J)+TRANS_MATRIX(J,NN)*ZA(NN)
   40       CONTINUE
   30 CONTINUE

      N=VAR
C      WRITE(*,*) ''
      
      END    
      
C ********************************************************************************** C 

      SUBROUTINE CROSS_PROD(X,Y,Z)
      
      DOUBLE PRECISION X(3),Y(3),Z(3),TEMP
      INTEGER I
      
      Z(1) = X(2)*Y(3) - X(3)*Y(2)
      Z(2) = -1 * (X(1)*Y(3) - X(3)*Y(1))
      Z(3) = X(1)*Y(2) - X(2)*Y(1)
      
      TEMP = SQRT(Z(1)**2 + Z(2)**2 + Z(3)**2)
      
      Z(1)=Z(1)/TEMP
      Z(2)=Z(2)/TEMP
      Z(3)=Z(3)/TEMP
      
C      WRITE(*,*) 'THE VALUE OF K ARE', K(1), K(2), K(3)
      
      END

C ********************************************************************************** C       

C----------------------------------------------------------------
C To evaluate the product A  of two 3x3 matrices B and C
C
      SUBROUTINE MAT_MUL_SIMILAR(RES,A,B,C)
      DOUBLE PRECISION A(3,3),B(3,3),C(3,3),TEMP(3,3)
      DOUBLE PRECISION RES(3,3)
      INTEGER I,J,K
      
      
      DO 10 I=1,3
         DO 20 J=1,3
            TEMP(I,J)=0
            RES(I,J)=0
   20    CONTINUE
   10 CONTINUE
      
      DO 30 I=1,3
         DO 40 J=1,3
            DO 50 K=1,3
               TEMP(J,I)=TEMP(J,I)+B(J,K)*C(K,I)
   50       CONTINUE
   40    CONTINUE
   30 CONTINUE
   
      DO 60 I=1,3
         DO 70 J=1,3
            DO 80 K=1,3
               RES(J,I)=RES(J,I)+A(J,K)*TEMP(K,I)
   80       CONTINUE
   70    CONTINUE
   60 CONTINUE
   
c      WRITE(*,*) 'NEW_MAT'
      
   
C
      RETURN
      END
C----------------------------------------------------------------      

C----------------------------------------------------------------
C To evaluate the product A  of two 3x3 matrices B and C
C
      SUBROUTINE MAT_MUL(A,B,C)
      DOUBLE PRECISION A(3,3),B(3,3),C(3,3)
      INTEGER I,J,K
      
      DO 10 I=1,3
         DO 20 J=1,3
            A(I,J)=0
   20    CONTINUE
   10 CONTINUE
   
      DO 30 I=1,3
         DO 40 J=1,3
            DO 50 K=1,3
               A(J,I)=A(J,I)+B(J,K)*C(K,I)
   50       CONTINUE
   40    CONTINUE
   30 CONTINUE
   
c      WRITE(*,*) 'NEW_MAT'
      
   
C
      RETURN
      END
C----------------------------------------------------------------      

C ********************************************************************************** C
      SUBROUTINE CAL_THETA(P,SIGMA,THETA,I)
     
      INTEGER DMAX
      
      PARAMETER (DMAX=24,PI=3.141592654)
      
      INTEGER I,J
      DOUBLE PRECISION P(3),SIGMA(3),THETA(DMAX),A

      
      A = 0.0
    
C      WRITE(*,*) 'SIGMA',SIGMA(1),SIGMA(2),SIGMA(3) 

      DO 20 J=1,3
          A = A + P(J) * SIGMA(J)
20    CONTINUE
     
      
      IF (A .GT. 1.0) THEN
         A=1.0
      ELSE IF(A .LT. -1.0) THEN
         A=-1.0
      END IF
      
      THETA(I)=ACOS(A)
    
      END
     
C ********************************************************************************* C 

C ********************************************************************************* C

      SUBROUTINE CRAMER(T,X,Y,Z,Q)
      
      DOUBLE PRECISION T(3),X(3),Y(3),Z(3),Q(3)
      INTEGER I,J
      DOUBLE PRECISION TEMPO(3,3),EQU(3,3)
      DOUBLE PRECISION DETER_EQU,DETER_TEMP
      
      DO 10 J=1,3
         EQU(1,J)=X(J)
         TEMPO(1,J)=X(J)
         EQU(2,J)=Y(J)
         TEMPO(2,J)=Y(J)
         EQU(3,J)=Z(J)
         TEMPO(3,J)=Z(J)
   10 CONTINUE
   
      DETER_EQU=EQU(1,1)*(EQU(2,2)*EQU(3,3) - EQU(2,3)*EQU(3,2))
&     +EQU(1,2)*(EQU(2,3)*EQU(3,1) - EQU(2,1)*EQU(3,3))
&     +EQU(1,3)*(EQU(2,1)*EQU(3,2) - EQU(2,2)*EQU(3,1))
              
      DO 20 I=1,3
         DO 30 J=1,3
            TEMPO(J,I)=Q(J)
   30    CONTINUE   
   
         DETER_TEMP=TEMPO(1,1)*(TEMPO(2,2)*TEMPO(3,3)-
&        TEMPO(2,3)*TEMPO(3,2))
&        +TEMPO(1,2)*(TEMPO(2,3)*TEMPO(3,1) - 
&        TEMPO(2,1)*TEMPO(3,3))
&        +TEMPO(1,3)*(TEMPO(2,1)*TEMPO(3,2) 
&        - TEMPO(2,2)*TEMPO(3,1))
                 
         T(I)=DETER_TEMP/DETER_EQU
         
         DO 40 J=1,3
            TEMPO(J,I)=EQU(J,I)
   40    CONTINUE  
   20 CONTINUE
   
      END
      
C ********************************************************************************* C

C ********************************************************************************* C

      SUBROUTINE CALCULATE_PHI(X,Y,PHI,I)
      
      DOUBLE PRECISION X(3),Y(3),PHI(24)
      INTEGER I
         
      PHI(I) = (X(1) * Y(1) + X(2) * Y(2) + X(3) * Y(3))
      
      IF(PHI(I) > 1) THEN
        PHI(I) = 1;
      ENDIF
      
      IF(PHI(I) < -1) THEN
        PHI(I) = -1;
      ENDIF
        
      PHI(I) = ACOS(PHI(I));
      
      END
      
C ********************************************************************************* C


C ********************************************************************************* C
      

      SUBROUTINE CALCULATE_MECH_ENG(X,Y,BETA,U,I,STRESS,SH,DI)
      
      DOUBLE PRECISION X(24),Y(24),U(24),STRESS,TAO,SIG
      DOUBLE PRECISION BETA,SH,DI
      INTEGER I

      
      SIG=DI*STRESS*COS(X(I))
      
      IF(X(I) .NE. 0) THEN    
        TAO=SH*STRESS*COS(BETA)*COS(Y(I))
      ELSE
         TAO=0.0
      END IF
      
      U(I)=TAO+SIG
      
c      WRITE(*,909) TAO,SIG,I
c909   format('shear, normal, variant',2f10.4,i5)
  
      RETURN
      
      END  
      
C ********************************************************************************* C

C ********************************************************************************* C

      SUBROUTINE SORTING(X,Y,Z,ALL_INFO)
      
      DOUBLE PRECISION X(24),Y(24),Z(24),ALL_INFO(24,4)
      INTEGER I,J
      DOUBLE PRECISION TEMPA,TEMPB,TEMPC,TEMPD
      
      DO 10 I=1,24
         ALL_INFO(I,1) = I
         ALL_INFO(I,2) = X(I)
         ALL_INFO(I,3) = Y(I)
         ALL_INFO(I,4) = Z(I)
   10 CONTINUE
   
      DO 20 I=1,24
         J=I
         DO 30 WHILE(J .LE. 24)
            IF(ALL_INFO(I,4) .LT. ALL_INFO(J,4)) THEN
            
              TEMPA=ALL_INFO(I,1)
              ALL_INFO(I,1)=ALL_INFO(J,1)
              ALL_INFO(J,1)=TEMPA
              
              TEMPB=ALL_INFO(I,2)
              ALL_INFO(I,2)=ALL_INFO(J,2)
              ALL_INFO(J,2)=TEMPB
              
              TEMPC=ALL_INFO(I,3)
              ALL_INFO(I,3)=ALL_INFO(J,3)
              ALL_INFO(J,3)=TEMPC
              
              TEMPD=ALL_INFO(I,4)
              ALL_INFO(I,4)=ALL_INFO(J,4)
              ALL_INFO(J,4)=TEMPD          
              J=I
            ELSE
              J=J+1
            ENDIF
            
   30    CONTINUE
   20 CONTINUE
   
      END
      
C **************************************************************************** C

C ******************** This subroutine calculates the inverse of a matrix ******

      SUBROUTINE CALCULATE_INVERSE(TATA,MAT_INV)   
      
      INTEGER I,J
      DOUBLE PRECISION DET,TATA(3,3)
      DOUBLE PRECISION MAT_INV(3,3)
      
      DET=TATA(1,1)*(TATA(2,2)*TATA(3,3)-TATA(2,3)*TATA(3,2))
&         -TATA(1,2)*(TATA(2,1)*TATA(3,3)-TATA(2,3)*TATA(3,1))
&         +TATA(1,3)*(TATA(2,1)*TATA(3,2)-TATA(2,2)*TATA(3,1))

      DO 10 I=1,3
         DO 20 J=1,3
            MAT_INV(I,J)=0
   20    CONTINUE
   10 CONTINUE

      MAT_INV(1,1)=(TATA(2,2)*TATA(3,3)-TATA(2,3)*TATA(3,2))/DET
      MAT_INV(1,2)=(TATA(3,2)*TATA(1,3)-TATA(3,3)*TATA(1,2))/DET
      MAT_INV(1,3)=(TATA(1,2)*TATA(2,3)-TATA(1,3)*TATA(2,2))/DET
      
      MAT_INV(2,1)=(TATA(2,3)*TATA(3,1)-TATA(2,1)*TATA(3,3))/DET
      MAT_INV(2,2)=(TATA(3,3)*TATA(1,1)-TATA(3,1)*TATA(1,3))/DET
      MAT_INV(2,3)=(TATA(1,3)*TATA(2,1)-TATA(1,1)*TATA(2,3))/DET
      
      MAT_INV(3,1)=(TATA(2,1)*TATA(3,2)-TATA(2,2)*TATA(3,1))/DET
      MAT_INV(3,2)=(TATA(3,1)*TATA(1,2)-TATA(3,2)*TATA(1,1))/DET
      MAT_INV(3,3)=(TATA(1,1)*TATA(2,2)-TATA(1,2)*TATA(2,1))/DET
      
      END
      
C ******************************End Subroutine *********************************     


C *****************  This subroutine calculates the TRANSPOSE of a matrix ******


      SUBROUTINE CALCULATE_TRANSPOSE(TRANS_MATRIX,TRANS_MATRIX_TRANSP) 
      
      DOUBLE PRECISION TRANS_MATRIX(3,3),TRANS_MATRIX_TRANSP(3,3)
      INTEGER I,J
      
      TRANS_MATRIX_TRANSP(1,1)=TRANS_MATRIX(1,1)
      TRANS_MATRIX_TRANSP(1,2)=TRANS_MATRIX(2,1)
      TRANS_MATRIX_TRANSP(1,3)=TRANS_MATRIX(3,1)
      
      TRANS_MATRIX_TRANSP(2,1)=TRANS_MATRIX(1,2)
      TRANS_MATRIX_TRANSP(2,2)=TRANS_MATRIX(2,2)
      TRANS_MATRIX_TRANSP(2,3)=TRANS_MATRIX(3,2)
      
      TRANS_MATRIX_TRANSP(3,1)=TRANS_MATRIX(1,3)
      TRANS_MATRIX_TRANSP(3,2)=TRANS_MATRIX(2,3)
      TRANS_MATRIX_TRANSP(3,3)=TRANS_MATRIX(3,3)
      
      END
      
C ******************************End Subroutine ********************************* 

C ************************ This is a subroutine ********************************

      SUBROUTINE CAL_DEFMAT(TRANS_MATRIX,TRANS_MATRIX_TRANSP,
&                 STRESS_TENSOR,DEF_MAT)
                 
      DOUBLE PRECISION TRANS_MATRIX(3,3),TRANS_MATRIX_TRANSP(3,3)
      DOUBLE PRECISION DEF_MAT(3,3),STRESS_TENSOR(3,3),TEMP(3,3)
      INTEGER H
      
      
      CALL MAT_MUL(TEMP,TRANS_MATRIX_TRANSP,STRESS_TENSOR)
      CALL MAT_MUL(DEF_MAT,TEMP,TRANS_MATRIX)
C      WRITE(*,*) 'DEF_MAT'
c      call writemat(TRANS_MATRIX)
c      call writemat(stress_tensor)   
c      call writemat(Trans_matrix_transp)
C      call writemat(DEF_MAT)
      RETURN
      
      END

C ******************************End Subroutine *********************************       
            subroutine writemat(a)
            double precision a(3,3)
            write(*,1) a(1,1),a(1,2),a(1,3),a(2,1),a(2,2),a(2,3)
&            ,a(3,1),a(3,2),a(3,3)
1           format(3f10.4/3f10.4/3f10.4//)
            return
            end
C ******************************End Subroutine *********************************     

      SUBROUTINE TENSOR_CALC(SIGMA,DEF_MAT,P,STRESS)      
      
      DOUBLE PRECISION SIGMA(3),DEF_MAT(3,3),P(3),STRESS
      DOUBLE PRECISION A
      INTEGER G,I,J,H
      
      SIGMA(1)=0.0
      SIGMA(2)=0.0
      SIGMA(3)=0.0
      
      DO 10 I=1,3
         DO 20 J=1,3
            SIGMA(I)=SIGMA(I)+DEF_MAT(I,J)*P(J)
20       CONTINUE
10    CONTINUE

      A=DSQRT(SIGMA(1)**2+SIGMA(2)**2+SIGMA(3)**2)
      
      STRESS=A
      
      DO 30 I=1,3
         SIGMA(I)=SIGMA(I)/A
30    CONTINUE
      
      IF(STRESS .EQ. 0) THEN
         SIGMA(1)=1
         SIGMA(2)=0
         SIGMA(3)=0
      END IF
      
C      WRITE(*,*) 'SIG',SIGMA(1),SIGMA(2),SIGMA(3),STRESS
C      DO 15 H=1,3
C            WRITE(*,995) DEF_MAT(H,1),DEF_MAT(H,2),DEF_MAT(H,3)
C995         FORMAT('SS',3D20.10)
C15      CONTINUE
C      WRITE(*,*) 'HABIT',HABIT(G,1),HABIT(G,2),HABIT(G,3)
      
C      STOP
      
      END
      
c ************************************************************************************ C

      SUBROUTINE GENERATE_RANDOM()
      
      DOUBLE PRECISION TEMP(3),TRANS(3,3),A,PI
      INTEGER I,GRAIN,ISEED,RX
      
      RX=77
      PI=3.14159
        
      OPEN(RX,FILE='random-matrix-input-plast.txt',STATUS='OLD')      
      
C
C ********* Generating the random number to get rotation axis and angle *******          
C        
      ISEED=TIME()
      CALL SRAND(ISEED)
      
      
      DO 10 GRAIN=1,1000    
        I=1
        DO 90 WHILE(I .LE. 3)
            A=RAND()

            IF(I .NE. 2) THEN
               TEMP(I)=2*PI*A
            ELSE
              TEMP(I)=2*A-1
            END IF 
            
            I=I+1
   90   CONTINUE

C
C ********************** Generating the rotational matrix ******************
C
   
C       WRITE(*,*) 'TEMP=',TEMP(1),TEMP(2),TEMP(3)
       
       TRANS(1,1) = COS(TEMP(1))*COS(TEMP(3))-SIN(TEMP(1))
&                            *TEMP(2)*SIN(TEMP(3))        
       TRANS(1,2) = SIN(TEMP(1))*COS(TEMP(3))+COS(TEMP(1))
&                            *TEMP(2)*SIN(TEMP(3)) 
       TRANS(1,3) = sqrt(1-TEMP(2)**2)*SIN(TEMP(3))
       
       TRANS(2,1) = -COS(TEMP(1))*SIN(TEMP(3))-SIN(TEMP(1))
&                            *TEMP(2)*COS(TEMP(3))
       TRANS(2,2) = -SIN(TEMP(1))*SIN(TEMP(3))+COS(TEMP(1))
&                            *TEMP(2)*COS(TEMP(3))
       TRANS(2,3) = sqrt(1-TEMP(2)**2)*COS(TEMP(3))
        
       TRANS(3,1) = SIN(TEMP(1))*sqrt(1-TEMP(2)**2)
       TRANS(3,2) = -COS(TEMP(1))*sqrt(1-TEMP(2)**2)
       TRANS(3,3) = TEMP(2)
      
       WRITE(77,*) TRANS(1,1)
       WRITE(77,*) TRANS(1,2)
       WRITE(77,*) TRANS(1,3)
      
       WRITE(77,*) TRANS(2,1)
       WRITE(77,*) TRANS(2,2)
       WRITE(77,*) TRANS(2,3)
      
       WRITE(77,*) TRANS(3,1)
       WRITE(77,*) TRANS(3,2)
       WRITE(77,*) TRANS(3,3)
       
       
   10 CONTINUE
   
      CLOSE(RX)
   
      END   

 